#!/usr/bin/env python
# -*- coding: UTF8 -*-

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# This file is part of system-config-vsftpd
#
# System-config-vsftpd is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# ( at your option ) any later version.
# 
# System-config-vsftpd is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Gnome-todo-manager; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

# Import GTK libraries
import gtk
import gettext
_ = gettext.gettext

def renderMainLabel( mainBox, ico, label, text ):

    """
        Render function for MainLabel showed on top of main window
    """

    # Label - one picture, two lines of text
    alignment = gtk.Alignment( 0.5, 0.5, 1.0, 1.0 )
    alignment.set_padding( 0, 0, 0, 10 )
    labelBox = gtk.HBox( False, 0 )
    mainBox.pack_start( labelBox, False, False, 0 )
    image = gtk.Image( )
    image.set_from_pixbuf( gtk.gdk.pixbuf_new_from_file( ico ) )
    alignment.add( image )
    label = gtk.Label( ' <b>'+label+'</b>\n '+text )
    label.set_alignment( 0, 0 )
    label.set_use_markup( 1 )
    labelBox.pack_start( alignment, False, False, 0 )
    labelBox.pack_start( label, False, False, 0 )
    label.show( )
    image.show( )
    alignment.show( )
    labelBox.show( )

    # horizontal separator
    mainBox.pack_start( renderSeparator( ), False, True, 0 )

def renderSeparator( box=None, padding=10 ):
    
    """
        Render visible separator ( line )
    """

    # render horizontal separator
    alignment = gtk.Alignment( 0.5, 0.5, 1.0, 1.0 )
    alignment.set_padding( int(padding/2), int(padding/2), 0, 0 )
    alignment.show( )
    separator = gtk.HSeparator( )
    alignment.add( separator )
    separator.show( )
    alignment.show( )

    if box != None: box.pack_start( alignment, False, True, 0 )

    return alignment

def renderVSeparator( box=None, padding=10 ):
    
    """
        Render visible separator ( line )
    """

    # render horizontal separator
    alignment = gtk.Alignment( 0.5, 0.5, 1.0, 1.0 )
    alignment.set_padding( 0, 0, int(padding/2), int(padding/2) )
    alignment.show( )
    separator = gtk.VSeparator( )
    alignment.add( separator )
    separator.show( )
    alignment.show( )

    if box != None: box.pack_start( alignment, False, True, 0 )

    return alignment

def renderFrame( label ):

    """
        Render frame with label as parameter
    """

    # render fram ewith label - label
    frame = gtk.Frame( )
    frame.set_label( '  '+label+'  ' )
    frame.set_label_align( 0, 0.5 )
    frame.get_label_widget( ).set_use_markup( True )
    frame.set_shadow_type( gtk.SHADOW_ETCHED_IN )
    frame.show( )
    
    #frame.get_alignment_widget( ).set_padding( 5, 5, 10, 0 )

    alignment = gtk.Alignment( 0.5, 0.5, 1.0, 1.0 )
    alignment.set_padding( 5, 5, 10, 0 )
    box = gtk.VBox( False, 0 )
    frame.add( alignment )
    alignment.add( box )
    alignment.show()
    box.show( )

    return frame, box

def renderAlignment( box, top, bottom, left, right, expand=False, fill=True ):

    """
        Render alignment - is using with other render functions
    """

    alignment = gtk.Alignment( 0.5, 0.5, 1.0, 1.0 )
    alignment.set_padding( top, bottom, left, right )
    if box != None: box.pack_start( alignment, expand, fill, 0 )
    alignment.show( )

    return alignment
    
def renderHBox( box, top, bottom, left, right ):

    """
        Render horizontal box with alignment
    """

    alignment = renderAlignment( box, top, bottom, left, right )
    box1 = gtk.HBox( False, 0 )
    alignment.add( box1 )
    box1.show( )

    if box == None: return (alignment, box1)
    return box1


def renderVBox( box, top, bottom, left, right ):

    """
        Render vertical box with alignment
    """

    alignment = renderAlignment( box, top, bottom, left, right )
    box1 = gtk.VBox( False, 0 )
    alignment.add( box1 )
    box1.show( )

    if box == None: return (alignment, box1)
    return box1

def renderButton( box, label, stock, underline, function ):

    """
        Render button with label
    """

    button = gtk.Button( label, stock, underline )
    button.show( )
    if function != None: button.connect( 'pressed', function )
    if box != None: box.pack_start( button, False, False, 0 )

    return button

def renderCheckButton( box, label, activity, functions ):

    """
        Render check button
    """

    chbutton = gtk.CheckButton( label, False )
    chbutton.set_active( activity )
    # bind mouse over function
    chbutton.show( )
    makeCconnect( chbutton, functions )
    if box != None: box.pack_start( chbutton, False, False, 0 )

    return chbutton

def renderRadioButton( box, group, label, toggled, functions ):

    """
        Render radio button
    """

    rbutton = gtk.RadioButton( group, label, toggled )
    rbutton.show( )
    makeCconnect( rbutton, functions )
    if box != None: box.pack_start( rbutton, False, False, 0 )

    return rbutton

def renderLabel( box, text, markup, paddingL, paddingR ):

    """
        Render label
    """

    label = gtk.Label( text )
    if box != None: box.pack_start( label, False, False, 0 )
    label.set_padding( paddingL, paddingR ) 
    label.set_alignment( 0, 0 )
    label.set_line_wrap( False )
    label.set_use_markup( markup )
    label.show( )

    return label

def renderEntry( box, max, functions ):

    """
        Render text entry with maximal length of string
    """

    entry = gtk.Entry( max )
    entry.show( )
    makeCconnect( entry, functions )
    if box != None: box.pack_start( entry, True, True, 0 )

    return entry

def renderSpin( box, value, lower, upper, step_incr, page_incr, page_size, climb_rate=0, digits=0, functions=None ):

    """
        Render spin button
    """

    adjustment = gtk.Adjustment( value, lower, upper, step_incr, page_incr, page_size )
    sButton = gtk.SpinButton( adjustment, climb_rate, digits )
    sButton.set_wrap( False )
    #sButton.set_size_request( 9, 20 )
    makeCconnect( sButton, functions )
    sButton.show( )
    if box != None: box.pack_start( sButton, False, False, 0 )

    return sButton

def renderTextView( box ):

    """
        Render text view
    """

    sw = gtk.ScrolledWindow( )
    sw.set_policy( gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC )
    textview = gtk.TextView( )
    textbuffer = textview.get_buffer( )
    sw.add( textview )
    sw.show( )
    textview.show( )
    textview.set_editable( False )
    textview.set_cursor_visible( True )

    if box != None: 
        box.pack_start( sw, True, True, 0 )
        return textview
    else:
        return textview, sw

def openFile( ):

    """
        Function used for file choooser dialog to select any file
    """

    file_open = gtk.FileChooserDialog(title="Select Configuration File"
			            , action=gtk.FILE_CHOOSER_ACTION_SAVE
			            , buttons=(gtk.STOCK_CANCEL
 		 	            , gtk.RESPONSE_CANCEL
			            , gtk.STOCK_OK
			            , gtk.RESPONSE_OK))
       
    """Create and add the 'all files' filter"""
    filter = gtk.FileFilter()
    filter.set_name("All files")
    filter.add_pattern("*")
    file_open.add_filter(filter)
    
    """Init the return value"""
    result = ""
    if file_open.run() == gtk.RESPONSE_OK:
        result = file_open.get_filename()
    file_open.destroy()
    if result != "": return result

def openDir( ):

    """
        Function used for file chooser dialog to select directory
    """

    dir_open = gtk.FileChooserDialog(title="Select Directory"
		                , action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER
			        , buttons=(gtk.STOCK_CANCEL
 		 	        , gtk.RESPONSE_CANCEL
			        , gtk.STOCK_OK
			        , gtk.RESPONSE_OK))
      
    
    """Init the return value"""
    result = ""
    if dir_open.run() == gtk.RESPONSE_OK:
        result = dir_open.get_filename()
    dir_open.destroy()
    if result != "": return result

def renderTable( box, tableModel ):
    
    """
        Function to render table from tableModel as parameter
    """

    # count rows and cols
    rowsCount = len ( tableModel )
    colsCount = 0
    for list in tableModel:
        if len ( list ) > colsCount: colsCount = len( list )

    table = gtk.Table( rowsCount, colsCount, False )
    rowP = -1
    for list in tableModel:
        rowP = rowP + 1
        lenList = len( list )
        colP = -1
        for obj in list:
            colP = colP + 1
            der = colsCount - lenList
            if colP == 0: 
                if obj: table.attach( obj, colP, colP+1+der, rowP, rowP+1, gtk.FILL )
            else: 
                if obj: table.attach( obj, colP+der, colP+1+der, rowP, rowP+1, gtk.FILL )

    if box != None: box.pack_start( table, False, False, 0 )
    table.show( )

def makeCconnect( widget, functions ):

    """
        Function to connect events to widgets
    """

    if type( widget ) == gtk.Entry:
        if functions[0] != None: widget.connect( 'changed', functions[0] )
        if functions[1] != None: widget.connect( 'enter-notify-event', functions[1] )
        if functions[2] != None: widget.connect( 'leave-notify-event', functions[2] )

    elif type( widget ) == gtk.SpinButton:
        if functions[0] != None: widget.connect( 'value_changed', functions[0] )
        if functions[1] != None: widget.connect( 'enter-notify-event', functions[1] )
        if functions[2] != None: widget.connect( 'leave-notify-event', functions[2] )

    else:
        if functions[0] != None: widget.connect( 'toggled', functions[0] )
        if functions[1] != None: widget.connect( 'enter-notify-event', functions[1] )
        if functions[2] != None: widget.connect( 'leave-notify-event', functions[2] )


def create_IP_List( box ):

    """
        Create ListView for files in Log
    """


    # create tree model of renderMenu ( left panel )
    liststore = gtk.ListStore( str )
    mainList = gtk.TreeView( liststore )
    cellTmp = gtk.CellRendererText( )
    temp = gtk.TreeViewColumn( "IP" )
    temp.pack_start( cellTmp, True )
    temp.set_attributes( cellTmp, text=0 )
    mainList.append_column( temp )

    mainList.show( )
    box.add_with_viewport( mainList )

    return liststore, mainList


def create_File_List( box ):

    """
        Create ListView for files in Log
    """


    # create tree model of renderMenu ( left panel )
    liststore = gtk.ListStore( str, gtk.gdk.Pixbuf, str, str )
    mainList = gtk.TreeView( liststore )

    cellTmp = gtk.CellRendererText( )
    temp = gtk.TreeViewColumn( "Time" )
    temp.pack_start( cellTmp, False )
    temp.set_attributes( cellTmp, text=0 )
    mainList.append_column( temp )

    cellTmp = gtk.CellRendererPixbuf( )
    temp = gtk.TreeViewColumn( "I/O" )
    temp.pack_start( cellTmp, False )
    temp.set_attributes( cellTmp, pixbuf=1 )
    mainList.append_column( temp )

    cellTmp = gtk.CellRendererText( )
    temp = gtk.TreeViewColumn( "User" )
    temp.pack_start( cellTmp, False )
    temp.set_attributes( cellTmp, text=2 )
    mainList.append_column( temp )

    cellTmp = gtk.CellRendererText( )
    temp = gtk.TreeViewColumn( "File" )
    temp.pack_start( cellTmp, True )
    temp.set_attributes( cellTmp, text=3 )
    mainList.append_column( temp )

    mainList.show( )
    box.pack_start( mainList, True, True, 0 )

    return liststore, mainList
