#!/usr/bin/env python
# -*- coding: UTF8 -*-

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# This file is part of system-config-vsftpd
#
# System-config-vsftpd is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# ( at your option ) any later version.
# 
# System-config-vsftpd is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Gnome-todo-manager; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 

# Import GTK and PyGTK libraries
import gtk
import gettext
_ = gettext.gettext

import string

from shell import Shell

import futils 
import logs

class renderMainWindow:
    
    """
        Class that describes functions for all render options
    """

    def die( self, parent ):

        """ Clean all rendered data """
        parent.mainBox.destroy()
        parent.scWindow.destroy()

        parent.scWindow = gtk.ScrolledWindow( )
        parent.scWindow.set_border_width( 0 )
        parent.scWindow.set_policy( gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC )
        parent.scWindow.show( )

        parent.mainBox = gtk.VBox( False, 0 )
        parent.xml_file.get_widget( "mainAlignment" ).add( parent.scWindow )

        alignment = gtk.Alignment( 0, 0, 1, 1 )
        alignment.set_padding( 10, 10, 10, 10 )
        alignment.add( parent.mainBox )
        alignment.show( )
       
        parent.scWindow.add_with_viewport( alignment )

    def show ( self, mainBox ):

        """ Show rendered data """

        mainBox.show( )

    def mouseOver( self, widget, value ):

        """ Called whhen mouse enter widget """

        if widget in self.mouseHints: self.appbar.set_status( self.mouseHints[widget] )

    def mouseEnd (self, widget, value ):

        """ Called, when mouse leave widget """

        self.appbar.set_status( '' )

class General( renderMainWindow ):

    """
        Class that describes functions for render "General" window
    """

    def __init__( self, mainBox, config, appbar ): 

        """ Constructor that render all widgets  """

        self.config = config
        self.startFlag = True
        self.appbar = appbar

        # function list for :
        # 0: when toggled
        # 1: when mouse over
        generalFunctions = [ self.change, self.mouseOver, self.mouseEnd ]

        # render main window from click on the left side

        #path="ico/"
        path="/usr/share/pixmaps/system-config-vsftpd/ico/"

        futils.renderMainLabel( mainBox, path+"monitorH.png", 'General', 
                'The following is a list of directives which control the overall behavior of the vsftpd daemon' )

        # --------------------------------------------------------------------------------------- #

        # CheckBox "Use TCP Wrappers
        self.chGeneralUseTCPWrappers = futils.renderCheckButton( mainBox, 'Use TCP wrappers ', False, generalFunctions )
	self.downloadEnable = futils.renderCheckButton( mainBox, 'Enable download files', True, generalFunctions )
	self.uploadEnable = futils.renderCheckButton( mainBox, 'Enable upload files', True, generalFunctions )

        # Separator
        futils.renderSeparator( mainBox )

        # Radio button "Run in standalone mode" with frame "Protocol Settings"
        self.rGeneralStandaloneMode = futils.renderRadioButton( mainBox, None, 'Run server in standalone mode', False, [ self.standalone_toggled ] + generalFunctions[1:] )

        # frame - Protocol settings
        frameBox = futils.renderHBox( mainBox, 5, 5, 25, 5 )
        [self.frame, box] = futils.renderFrame( 'Protocol Settings' )
        frameBox.pack_start( self.frame, True, True, 0 )

        # box for ComboBoxEntry with IPv4 or IPv6 option # TODO - make function for render ComboBox
        box1 = futils.renderHBox( box, 0, 8, 0, 0 )
        comboModel = gtk.ListStore( str )
        self.cbGeneralStandaloneProtocol = gtk.ComboBoxEntry( comboModel )
        self.cbGeneralStandaloneProtocol.show( )
        comboText = gtk.CellRendererText( )
        self.cbGeneralStandaloneProtocol.pack_start( comboText, True )
        comboModel.append( ['IPv4'] )
        comboModel.append( ['IPv6'] )
        box1.pack_start( self.cbGeneralStandaloneProtocol, True, False, 0 )

        # Radio button "Do not run in standalone mode" i group with "Run in standalone mode"
        self.rGeneralNotStandaloneMode = futils.renderRadioButton( mainBox, self.rGeneralStandaloneMode, 'Do not run in standalone mode', False, [ None] + generalFunctions[1:] )
        self.cbGeneralStandaloneProtocol.connect( "changed", self.change )

        self.fill()
        self.startFlag = False

        self.mouseHints = { 
             self.chGeneralUseTCPWrappers     : 'Use TCP wrappers to grant access to the server.',
             self.downloadEnable              : 'Enable download files from server',
             self.uploadEnable                : 'Enable upload files. ( see in Users -> Anonymous users -> Allow upload files )',
             self.rGeneralStandaloneMode      : 'Run vsftpd in standalone mode.',
              self.rGeneralNotStandaloneMode   : 'Do not run in standalone mode.'
        }

        self.show( mainBox )

    def fill( self ):

        """ Function to fill widgets with values from configuration  """
        self.chGeneralUseTCPWrappers.set_active( self.config["tcp_wrappers"] == "YES" )
        self.downloadEnable.set_active( self.config["download_enable"] == "YES" )
        self.uploadEnable.set_active( self.config["write_enable"] == "YES" )
        self.cbGeneralStandaloneProtocol.set_active( self.config["listen_ipv6"] == "YES" ) 
        self.rGeneralNotStandaloneMode.set_active( self.config["listen"] == "NO" and self.config["listen_ipv6"] == "NO" )
        #print self.config["listen"] + self.config["listen_ipv6"]

    def standalone_toggled( self, widget ):

        """Called when one of standalone radiobuttons toggled"""
        self.frame.set_sensitive( self.rGeneralStandaloneMode.get_active( ) )
        self.cbGeneralStandaloneProtocol.set_sensitive( self.rGeneralStandaloneMode.get_active( ) )

        #print "TOGGLE %d" % self.rGeneralStandaloneMode.get_active( )

        # Change settings in config
        if self.rGeneralStandaloneMode.get_active( ) == False: 
            self.config["listen"] = "NO"
            self.config["listen_ipv6"] = "NO"
        elif self.cbGeneralStandaloneProtocol.get_active_text( ) == "IPv4":
            self.config["listen"] = "YES"
            self.config["listen_ipv6"] = "NO"
        else: 
            self.config["listen"] = "NO"
            self.config["listen_ipv6"] = "YES"

    def change( self, widget ):

        """ Function called when user change some value on the form """
        if self.startFlag: return
        if   widget == self.downloadEnable:  
		if self.downloadEnable.get_active( ): self.config["download_enable"] = "YES"
		else: self.config["download_enable"] = "NO"
        elif widget == self.uploadEnable:  
		if self.uploadEnable.get_active( ): self.config["write_enable"] = "YES"
		else: self.config["write_enable"] = "NO"
        elif widget == self.chGeneralUseTCPWrappers:
            if widget.get_active( ) == True: self.config["tcp_wrappers"] = "YES"
            else: self.config["tcp_wrappers"] = "NO"
        elif widget == self.cbGeneralStandaloneProtocol:
            #print "CHANGE %d" % self.rGeneralStandaloneMode.get_active( )
            if self.cbGeneralStandaloneProtocol.get_active_text( ) == "IPv4" and self.rGeneralStandaloneMode.get_active( ):
                self.config["listen"] = "YES"
                self.config["listen_ipv6"] = "NO"
            elif self.rGeneralStandaloneMode.get_active( ): 
                self.config["listen"] = "NO"
                self.config["listen_ipv6"] = "YES"
            else:
                self.config["listen"] = "NO"
                self.config["listen_ipv6"] = "NO"


class ServerControl( renderMainWindow ):

    """
        Class that describes functions for render "Server Control" window
    """

    def __init__( self, mainBox, config ): 

        #path="ico/"
        path="/usr/share/pixmaps/system-config-vsftpd/ico/"

        """ Constructor that render all widgets  """
        self.config = config
        self.startFlag = True
        self.shell = Shell( self.config )

        self.logNotebook = gtk.Notebook( )
        self.logNotebook.set_tab_pos( gtk.POS_TOP )
        self.logNotebook.show( )

        # Labels for notebook
        slabel = gtk.Label( 'Status Log' )
        slabel.show( )
        llabel = gtk.Label( 'Standard Log' )
        llabel.show( )
        xlabel = gtk.Label( 'XFerlog' )
        xlabel.show( )
        ylabel = gtk.Label( 'System Log' )
        ylabel.show( )


        # render main window from click on the left side
        futils.renderMainLabel( mainBox, path+"serverH.png", 'Server Control', 'Service control management' )

        # --------------------------------------------------------------------------------------- #

        # frame - Server Status
        [frame, box] = futils.renderFrame( 'Server status' )
        mainBox.pack_start( frame, False, True, 0 )

        # box for ComboBoxEntry with IPv4 or IPv6 option
        box1 = futils.renderHBox( box, 0, 10, 0, 0 )
        box2 = futils.renderVBox( box1, 0, 0, 0, 0 )
        box3 = futils.renderHBox( box1, 15, 15, 100, 0 )

        self.lServerStatus = futils.renderLabel( box2, '', True, 5, 0 )
        self.lServerAction = futils.renderLabel( box2, '', True, 10, 10 )
        self.bServerAction = futils.renderButton( box1, '', None, True, self.status_toggled )

        self.status( )

        # horizontal separator
        futils.renderHBox( mainBox, 5, 5, 0, 0 )
        # Separator
               
        mainBox.pack_start( self.logNotebook, True, True, 0 )

        # Box for rendering system box
        box = gtk.VBox( False, 0 ) 
        box.show( )
        alignment = futils.renderAlignment( box, 10, 10, 10, 10, True, True )
        self.statusBox = gtk.VBox( False, 0 )
        alignment.add( self.statusBox )
        self.statusBox.show( )
        self.logNotebook.append_page( box, slabel )

        futils.renderLabel( self.statusBox, "<i>This log shows all messages during server startup or stop</i>", True, 0, 0 )

        # horizontal separator
        futils.renderHBox( self.statusBox, 5, 5, 0, 0 )

        self.twServerStatus = futils.renderTextView( self.statusBox )

        # --------------------------------------------------------------------------------------------

        # Box for rendering system box
        box = gtk.VBox( False, 0 ) 
        box.show( )
        alignment = futils.renderAlignment( box, 10, 10, 10, 10, True, True )
        self.logBox = gtk.VBox( False, 0 )
        alignment.add( self.logBox )
        self.logBox.show( )
        self.logNotebook.append_page( box, llabel )

        futils.renderLabel( self.logBox, "<i>This log shows all standard logs from vsftpd</i>", True, 0, 0 )

        # horizontal separator
        futils.renderHBox( self.logBox, 5, 5, 0, 0 )

        self.twServerLog = futils.renderTextView( self.logBox )

        # --------------------------------------------------------------------------------------------

        # Box for rendering system box
        box = gtk.VBox( False, 0 ) 
        box.show( )
        alignment = futils.renderAlignment( box, 10, 10, 10, 10, True, True )
        self.xferlogBox = gtk.VBox( False, 0 )
        alignment.add( self.xferlogBox )
        self.xferlogBox.show( )
        self.logNotebook.append_page( box, xlabel )

        futils.renderLabel( self.xferlogBox, "<i>Wu-ftpd compatible log</i>", True, 0, 0 )

        # horizontal separator
        futils.renderHBox( self.xferlogBox, 5, 5, 0, 0 )

        self.twServerXLog = futils.renderTextView( self.xferlogBox )

        # --------------------------------------------------------------------------------------------

        # Box for rendering system box
        box = gtk.VBox( False, 0 ) 
        box.show( )
        alignment = futils.renderAlignment( box, 10, 10, 10, 10, True, True )
        self.sysBox = gtk.VBox( False, 0 )
        alignment.add( self.sysBox )
        self.sysBox.show( )
        self.logNotebook.append_page( box, ylabel )

        futils.renderLabel( self.sysBox, "<i>System log shows all logs from //log/messages which contains 'vsftpd'</i>", True, 0, 0 )

        # horizontal separator
        futils.renderHBox( self.sysBox, 5, 5, 0, 0 )

        self.twServerSysLog = futils.renderTextView( self.sysBox )

        self.logs( )
        self.show( mainBox )

    def logs( self ):

        """ Function to load logs """
        stdlog, xferlog, syslog = self.shell.getLogs( )
        self.twServerLog.get_buffer( ).set_text( stdlog )
        self.twServerXLog.get_buffer( ).set_text( xferlog )
        self.twServerSysLog.get_buffer( ).set_text( syslog )

    def status( self ):

        """ Function to check server status """
        self.shell.__init__( self.config )
        act = self.shell.getActuall( )
        if act == '1': client = "client"
        else: client = "clients"

        # server is running
        if self.shell.status[0]: 
            self.lServerStatus.set_label( '<b>Server is stopped</b>' )
            self.lServerAction.set_label( 'Click this button to start ftp server.' )
            self.bServerAction.set_label( '         Start FTP Server          ' )
            self.bServerAction.set_image( gtk.image_new_from_stock( 'gtk-media-play', 3 ) )
        else:
            self.lServerStatus.set_label( '<b>Server is running</b> ( '+act+' '+client+')' )
            self.lServerAction.set_label( 'Click this button to stop ftp server.')
            self.bServerAction.set_label( '          Stop FTP Server          ' )
            self.bServerAction.set_image( gtk.image_new_from_stock( 'gtk-stop', 3 ) )


    def status_toggled( self, widget ):

        """ Function called, wwhen user clicked on start/stop button"""
        # server is not running
        if self.shell.status[0]: 
            message = self.shell.startServer( )
            #start = string.find( message, ":" ) + 2
            #firstSpace = string.find( message, " ", start )
            #secondSpace = string.find( message, " ", firstSpace + 2 )
            #message = message[ 0 : start ] + message[ firstSpace : secondSpace ]
            self.twServerStatus.get_buffer( ).insert( self.twServerStatus.get_buffer( ).get_end_iter(), message+'\n' )
            self.status( )

        else:
            message = self.shell.stopServer( )
            #start = string.find( message, ":" ) + 2
            #firstSpace = string.find( message, " ", start )
            #secondSpace = string.find( message, " ", firstSpace + 2 )
            #message = message[ 0 : start ] + message[ firstSpace : secondSpace ]
            self.twServerStatus.get_buffer( ).insert( self.twServerStatus.get_buffer( ).get_end_iter(), message+'\n' )
            self.status( )

        self.logs( )

class AccessControl( renderMainWindow ):

    """
        Class that describes functions for render "Access Control" window
    """

    def __init__( self, mainBox, config, appbar ): 

        """ Constructor that render all widgets  """
        self.config = config
        self.appbar = appbar

        generalFunctions = [ self.change, self.mouseOver, self.mouseEnd ]

        #path="ico/"
        path="/usr/share/pixmaps/system-config-vsftpd/ico/"

        # render main window from click on the left side
        futils.renderMainLabel( mainBox, path+"accessH.png", 'Access Control', 'The following is a list of directives which control the login behavior and access control mechanisms' )

        # ------------------------------------------------------------------------------------- #
        # frame with label - Banners

        [frame, box] = futils.renderFrame( 'Banners' )
        mainBox.pack_start( frame, False, True, 0 )

        # Access Banner String ( label with entry )
        box1 = futils.renderHBox( box, 0, 0, 0, 10 )

        self.rAccessBannerString = futils.renderRadioButton( box1, None, 'Use string : ', False, [ None ] + generalFunctions[1:] )
        self.eAccessBannerString = futils.renderEntry( box1, 0, generalFunctions )

        # Access Banner File box ( label with entry and button )
        box1 = futils.renderHBox( box, 0, 0, 0, 10 )

        self.rAccessBannerFile = futils.renderRadioButton( box1, self.rAccessBannerString, 'Use file :     ', False, [ self.banner_toggled ] + generalFunctions[1:] )
        self.eAccessBannerFile = futils.renderEntry( box1, 0, generalFunctions )
        self.bAccessBannerFile = futils.renderButton( box1, 'Change...', None, False, self.change )

        # Information text above Access Banner File box
        box1 = futils.renderHBox( box, 5, 5, 5, 0 )
        self.lAccessBanner = futils.renderLabel( box1, 'Text displayed when a connection is established to the server.\n'
                                                       'Eg. Welcome to blah ftp service.', False, 0, 0 )

        # horizontal separator
        futils.renderHBox( mainBox, 5, 5, 0, 0 )

        # ------------------------------------------------------------------------------------- #
        # frame with label - Email bans
        [frame, box] = futils.renderFrame( 'Email bans' )
        mainBox.pack_start( frame, False, True, 0 )

        # check box: define file where are writed denied emails
        box1 = futils.renderHBox( box, 0, 0, 0, 10 )

        # CheckButton 
        self.chAccessEmailBansFile = futils.renderCheckButton( box1, 'Define emails not permitted access to the server ', True, [ self.emailBans_toggled ] + generalFunctions[1:] )

        # Entry with box with path to "emails not permitted.. " file
        box1 = futils.renderHBox( box, 0, 10, 10, 10 )
        self.eAccessEmailBansFile = futils.renderEntry( box1, 0, generalFunctions )
        self.bAccessEmailBansFile = futils.renderButton( box1, 'Change...', None, False, self.change )

        # horizontal separator
        futils.renderHBox( mainBox, 5, 5, 0, 0 )

        # ------------------------------------------------------------------------------------- #
        # frame with label - Local Users
        [frame, box] = futils.renderFrame( 'Local users' )
        mainBox.pack_start( frame, False, True, 0 )

        # check box
        box1 = futils.renderHBox( box, 0, 0, 0, 10 )
        self.chAccessEnableUserList = futils.renderCheckButton( box1, 'Enable list of allowed / denied users ', False, [ self.enableUserList_toggled ] + generalFunctions[1:] )

        # check box
        box1 = futils.renderHBox( box, 0, 0, 0, 10 )
        self.chAccessEnableDeny = futils.renderCheckButton( box1, 'Deny users in the list (otherwise allow) ', False, [ self.enableDenyList_toggled ] + generalFunctions[1:] )

        # second box
        box1 = futils.renderHBox( box, 0, 10, 10, 10 )
        self.eAccessUserListFile = futils.renderEntry( box1, 0, generalFunctions )
        self.bAccessUserListFile = futils.renderButton( box1, 'Change...', None, False, self.change )

        self.fill( )
        self.enableUserList_toggled( None )
        self.banner_toggled( None )

        self.mouseHints = { 
             self.rAccessBannerString    : 'Specified string is displayed when a connection is established to the server.',
             self.eAccessBannerString    : 'Specified string is displayed when a connection is established to the server.',
             self.rAccessBannerFile      : 'Specified file containing text displayed when a connection is established to the server.',
             self.eAccessBannerFile      : 'Specified file containing text displayed when a connection is established to the server.',
             self.chAccessEmailBansFile  : 'This directive specifies the file containing a list of anonymous email passwords which are not permitted to the server.',
             self.eAccessEmailBansFile   : 'This file containing a list of anonymous email passwords which are not permitted to the server.',
             self.chAccessEnableUserList : 'When enabled the users listed in the file below are denied or allowed access to the server ( see Deny users directive below ).',
             self.chAccessEnableDeny     : 'When enabled the users listed in the file below are DENIED to access to the server, otherwise ALLOWED',
             self.eAccessUserListFile    : 'This file containing a list of users which are allowed or denied to access to the server depends on directive above'
        }

        self.show( mainBox )

    def change( self, widget ):

        """ Function called when user change some value on the form """
        choice = None

        if   widget == self.eAccessBannerString:    self.config["ftpd_banner"] = self.eAccessBannerString.get_text( )
        elif widget == self.eAccessBannerFile:    self.config["banner_file"] = self.eAccessBannerFile.get_text( )
        elif widget == self.eAccessEmailBansFile: self.config["banned_email_file"] = self.eAccessEmailBansFile.get_text( )
        elif widget == self.eAccessUserListFile:  self.config["userlist_file"] = self.eAccessUserListFile.get_text( )

        else: choice = futils.openFile( )

        if choice:
            if   widget == self.bAccessBannerFile:    self.eAccessBannerFile.set_text( choice )
            elif widget == self.bAccessEmailBansFile: self.eAccessEmailBansFile.set_text( choice )
            elif widget == self.bAccessUserListFile:  self.eAccessUserListFile.set_text( choice )

    def fill( self ):

        """ Function to fill widgets with values from configuration  """
        if len(self.config["ftpd_banner"] ) > 0: self.eAccessBannerString.set_text( self.config["ftpd_banner"] )
        if len(self.config["banner_file"] ) > 0:
            self.eAccessBannerFile.set_text( self.config["banner_file"] )
            self.rAccessBannerFile.set_active( True )

        self.chAccessEmailBansFile.set_active( self.config["deny_email_enable"] == "YES" )
        self.eAccessEmailBansFile.set_text( self.config["banned_email_file"] )

        self.chAccessEnableUserList.set_active( self.config["userlist_enable"] == "YES" )
        self.chAccessEnableDeny.set_active( self.config["userlist_deny"] == "YES" )
        self.eAccessUserListFile.set_text( self.config["userlist_file"] )

    def banner_toggled( self, widget ):

        """Called when radiobuttons in banner site toggled"""
        self.eAccessBannerString.set_sensitive( self.rAccessBannerString.get_active( ) )
        self.eAccessBannerFile.set_sensitive( self.rAccessBannerFile.get_active( ) )
        self.bAccessBannerFile.set_sensitive( self.rAccessBannerFile.get_active( ) )

        # Set Config
        if self.rAccessBannerFile.get_active( ) == True: 
            self.config["ftpd_banner"] = ""
            self.config["banner_file"] = self.eAccessBannerFile.get_text( )
        else: 
            self.config["banner_file"] = ""
            self.config["ftpd_banner"] = self.eAccessBannerString.get_text( )


    def emailBans_toggled( self, widget ):

        """Called when toggled checkbutton on Email Bans"""
        self.eAccessEmailBansFile.set_sensitive( self.chAccessEmailBansFile.get_active( ) )
        self.bAccessEmailBansFile.set_sensitive( self.chAccessEmailBansFile.get_active( ) )

        # Set Config
        if self.chAccessEmailBansFile.get_active( ) == True: 
            self.config["deny_email_enable"] = "YES"
            self.config["banned_email_file"] = self.eAccessEmailBansFile.get_text( )
        else: self.config["deny_email_enable"] = "NO"
        

    def enableUserList_toggled( self, widget ):

        """Called when radiobuttons in enable user list toggled"""
        self.chAccessEnableDeny.set_sensitive( self.chAccessEnableUserList.get_active( )  )
        self.eAccessUserListFile.set_sensitive( self.chAccessEnableUserList.get_active( ) )
        self.bAccessUserListFile.set_sensitive( self.chAccessEnableUserList.get_active( ) )

        # Set Config
        if self.chAccessEnableUserList.get_active( ) == True: self.config["userlist_enable"] = "YES"
        else: self.config["userlist_enable"] = "NO"

    def enableDenyList_toggled( self, widget ):

        """Called when checkbox on Enable deny list get toggled"""
        if self.chAccessEnableDeny.get_active( ) == True: self.config["userlist_deny"] = "YES"
        else: self.config["userlist_deny"] = "NO"

class UsersControl( renderMainWindow ):

    """
        Class that describes functions for render "Users" window
    """

    def __init__( self, mainBox, config, appbar ): 

        """ Constructor that render all widgets  """
        self.config = config
        self.appbar = appbar
        generalFunctions = [ self.change, self.mouseOver, self.mouseEnd ]

        #path="ico/"
        path="/usr/share/pixmaps/system-config-vsftpd/ico/"

        # render main window from click on the left side
        futils.renderMainLabel( mainBox, path+"usersH.png", 'Users', 
                'the following is a list of directives which charakterize the way local and anonymous users access the server.' )

        self.usersNotebook = gtk.Notebook( )
        self.usersNotebook.set_tab_pos( gtk.POS_TOP )
        self.usersNotebook.show( )

        mainBox.pack_start( self.usersNotebook, True, True, 0 )

        # Labels for notebook
        nlabel = gtk.Label( 'System users' )
        nlabel.show( )
        alabel = gtk.Label( 'Anonymous user' )
        alabel.show( )
        virtualLabel = gtk.Label( 'Virtual users' )
        virtualLabel.show( )

        # ------------------------------------------------------------------------------------- #
        # Local users

        # Box for rendering system box
        box = gtk.VBox( False, 0 )
        box.show( )
        self.usersBox = futils.renderVBox( box, 10, 10, 10, 10 )
        self.usersNotebook.insert_page( box, nlabel, 1 )
        
        # [local_enable]
        self.chAllowLocalUsers = futils.renderCheckButton( self.usersBox, 'Allow local users', True, [self.allowLocalUsers] + generalFunctions[1:] )

        # Separator
        futils.renderSeparator( self.usersBox )

        # Render box for user options
        futils.renderHBox( self.usersBox, 5, 5, 0, 0 )
        self.systemUsersBox = futils.renderVBox( self.usersBox, 0, 0, 0, 0 )
        
        # [chmod_enable]
        self.chUsersChmodEnable = futils.renderCheckButton( self.systemUsersBox, 'Allow users to change the permissions on files', False, generalFunctions )

        # [chroot_local_user]
        self.chUsersChangeRoot = futils.renderCheckButton( self.systemUsersBox, 'Change root directory for the users to their home directories', False, generalFunctions )

        # [?]
        self.chPasswdChRootEnable = futils.renderCheckButton( self.systemUsersBox, 'Change root directory based on the occurece of the "." in the home '
                                                                                   'directory field within /etc/passwd', False, generalFunctions )
        # Separator
        futils.renderSeparator( self.systemUsersBox )
               
        # frame with label
        [frame, box] = futils.renderFrame( 'Chroot Jail' )
        self.systemUsersBox.pack_start( frame, False, True, 0 )
        # check box: define file where are writed..
        box1 = futils.renderHBox( box, 0, 0, 0, 10 )
        # [chroot_list_enable]
        self.chRootListEnable = futils.renderCheckButton( box1, 'Place only users from list below to "chroot jail"', False, [self.enableRootList_toggled] + generalFunctions[1:] )
        # Entry with box with path to file
        box1 = futils.renderHBox( box, 0, 10, 10, 10 )
        # [chroot_list_file]
        self.eRootListEnable = futils.renderEntry( box1, 0, generalFunctions )
        self.bRootListEnable = futils.renderButton( box1, 'Change...', None, False, self.change )
        # horizontal separator
        futils.renderHBox( self.systemUsersBox, 5, 5, 0, 0 )

        # frame with label
        [frame, box] = futils.renderFrame( 'Guest' )
        self.systemUsersBox.pack_start( frame, False, True, 0 )
        # check box: define file where are writed..
        box1 = futils.renderHBox( box, 0, 0, 0, 10 )
        # [guest_enable]
        self.chGuestEnable = futils.renderCheckButton( box1, 'Change all non-anonymous users logged in to the user:', False, [self.guestEnable_toggled] + generalFunctions[1:] )
        # Entry with box with path to file
        box1 = futils.renderHBox( box, 0, 10, 10, 10 )
        # [guest_username]
        self.eGuestEnable = futils.renderEntry( box1, 0, generalFunctions )

        # horizontal separator
        futils.renderHBox( self.systemUsersBox, 5, 5, 0, 0 )

        # [local_root]
        self.lLocalRoot = futils.renderLabel( box, 'Specify the directory <i>vsftpd</i> changes to after a local user logs in:', True, 0, 0 )
        # Entry with box with path to file
        box1 = futils.renderHBox( box, 0, 10, 10, 10 )
        self.eLocalRoot = futils.renderEntry( box1, 0, generalFunctions )
        self.bLocalRoot = futils.renderButton( box1, 'Change...', None, False, self.change )

        # horizontal separator
        futils.renderHBox( self.systemUsersBox, 5, 5, 0, 0 )

        # [user_config_dir]
        self.lUserConfigDir = futils.renderLabel( box, 'Specify the path to a directory containing configuration files bearing the name of local system\n'
                                                    'users that contain specific settings for that user:', True, 0, 0 )
        # Entry with box with path to file
        box1 = futils.renderHBox( box, 0, 10, 10, 10 )
        self.eUserConfigDir = futils.renderEntry( box1, 0, generalFunctions )
        self.bUserConfigDir = futils.renderButton( box1, 'Change...', None, False, self.change )

        # Separator
        futils.renderSeparator( self.systemUsersBox )
        # horizontal separator
        futils.renderHBox( self.systemUsersBox, 5, 5, 0, 0 )
               
        box = futils.renderHBox( self.systemUsersBox, 0, 0, 0, 0 )

        label = futils.renderLabel( None, 'Maximum data transfer rate (B/s):   ', False, 0, 0 ) 
        label.set_alignment( 0, 0.5 )
        # [local_max_rate]
        self.sLocalMaxRate = futils.renderSpin( None, 0.0, 0.0, 999999999.0, 1.0, 100.0, 0.0, 0, 0, generalFunctions )

        # [local_umask]
        self.lLocalUmask = futils.renderLabel( None, 'Umask for file creation: ', False, 0, 0 )
        self.lLocalUmask.set_alignment( 0, 0.5 )
        self.eLocalUmask = futils.renderEntry( None, 3, generalFunctions )

        tableModel = [ [ label, self.sLocalMaxRate ],
                       [ self.lLocalUmask, self.eLocalUmask ] ]

        futils.renderTable( box, tableModel )

        # ------------------------------------------------------------------------------------- #
        # Anonymous user

        # Box for rendering system box
        box = gtk.VBox( False, 0 ) 
        box.show( )
        self.anonymousBox = futils.renderVBox( box, 10, 10, 10, 10 )
        self.usersNotebook.append_page( box, alabel )

        # [anonymous_enable]
        self.chAllowAnonymousUser = futils.renderCheckButton( self.anonymousBox, 'Allow anonymous user', True, [self.allowAnonymousUser] + generalFunctions[1:] )
        
        # Separator
        futils.renderSeparator( self.anonymousBox )

        # Render box for anonymous options
        futils.renderHBox( self.anonymousBox, 5, 5, 0, 0 )
        self.anonymousUserBox = futils.renderVBox( self.anonymousBox, 0, 0, 0, 0 )

        # [anon_mkdir_write_enable]
        self.chAllowCreatingNewDirectories = futils.renderCheckButton( self.anonymousUserBox, 'Allow creating new directories', False, generalFunctions ) 

        # [anon_upload_enable]
        self.chAllowUploadFiles = futils.renderCheckButton( self.anonymousUserBox, 'Allow upload files', False, generalFunctions ) 

        # [no_anon_password]
        self.chAskForPassword = futils.renderCheckButton( self.anonymousUserBox, 'Do not ask for a password', False, generalFunctions ) 

        # [anon_world_readable_only]
        box1 = futils.renderHBox( self.anonymousUserBox, 0, 0, 0, 0 )
        self.chAnonymousWorldOnlyEnable = futils.renderCheckButton( box1, 'Allow to download only world-readable files', False, generalFunctions )

        # [allow_anon_ssl]
        box1 = futils.renderHBox( self.anonymousUserBox, 0, 0, 0, 0 )
        self.chAnonymousUseSSL = futils.renderCheckButton( box1, 'Allow anonymous to use ssl connection', False, generalFunctions )

        # Separator
        futils.renderSeparator( self.anonymousUserBox )
               
        # frame with label
        [frame, box] = futils.renderFrame( 'Changing owner' )
        self.anonymousUserBox.pack_start( frame, False, True, 0 )
        # check box: define file where are writed..
        box2 = futils.renderHBox( box, 0, 0, 0, 10 )
        # [chown_uploads]
        self.chAnonymousChOwner = futils.renderCheckButton( box2, 'Change owner on all uploaded files to user below ', False, [self.chownUploads_toggled] + generalFunctions[1:] )
        # Entry with box with path to file
        box2 = futils.renderHBox( box, 0, 10, 10, 10 )
        self.eAnonymousChOwner = futils.renderEntry( box2, 0, generalFunctions )

        # horizontal separator
        futils.renderHBox( self.anonymousUserBox, 5, 5, 0, 0 )
        box = futils.renderHBox( self.anonymousUserBox, 0, 0, 0, 0 )

        self.lAnonymousUserName = futils.renderLabel( None, 'FTP user:   ', False, 0, 0 ) 
        self.lAnonymousUserName.set_alignment( 0, 0.5 )
        # [ftp_username]
        self.eAnonymousUserName = futils.renderEntry( None, 30, generalFunctions )
        
        (alignment1, box1) = futils.renderVBox( None, 5, 5, 0, 0 )

        self.lAnonymousRootDir = futils.renderLabel( None, 'Root directory:   ', False, 0, 0 ) 
        self.lAnonymousRootDir.set_alignment( 0, 0.5 )
        # [anon_root]
        self.eAnonymousRootDir = futils.renderEntry( None, 30, generalFunctions )

        self.bAnonymousRootDir = futils.renderButton( None, ' Change... ', None, False, self.change )
        alignment2 = gtk.Alignment( 0, 0.5, 0, 0 )
        alignment2.add( self.bAnonymousRootDir )
        alignment2.show( )

        tableModel = [ [ self.lAnonymousUserName, self.eAnonymousUserName, None ],
                       [ alignment1 ],
                       [ self.lAnonymousRootDir, self.eAnonymousRootDir, alignment2 ] ]


        futils.renderTable( box, tableModel )

        # horizontal separator
        futils.renderHBox( self.anonymousUserBox, 5, 5, 0, 0 )
        # Separator
        futils.renderSeparator( self.anonymousUserBox )
        # horizontal separator
        futils.renderHBox( self.anonymousUserBox, 5, 5, 0, 0 )
               
        box = futils.renderHBox( self.anonymousUserBox, 0, 0, 0, 0 )

        # [anon_max_rate]
        self.lAnonymousMaxRate = futils.renderLabel( None, 'Maximum data transfer rate (B/s): ', False, 0, 0 )
        self.lAnonymousMaxRate.set_alignment( 0, 0.5 )
        self.sAnonymousMaxRate = futils.renderSpin( None, 0.0, 0.0, 999999999.0, 1.0, 100.0, 0.0, 0, 0, generalFunctions )
        
        # [anon_umask]
        self.lAnonUmask = futils.renderLabel( None, 'Umask for file creation: ', False, 0, 0 )
        self.lAnonUmask.set_alignment( 0, 0.5 )
        self.eAnonUmask = futils.renderEntry( None, 3, generalFunctions )

        tableModel = [ [ self.lAnonymousMaxRate, self.sAnonymousMaxRate ],
                       [ self.lAnonUmask, self.eAnonUmask ] ]

        futils.renderTable( box, tableModel )


        # ------------------------------------------------------------------------------------- #
        # Virtual users

        # Box for rendering system box
        box = gtk.VBox( False, 0 ) 
        box.show( )
        self.virtualBox = futils.renderVBox( box, 10, 10, 10, 10 )
        #self.usersNotebook.append_page( box, virtualLabel )

        # [anonymous_enable]
        self.chAllowVirtualUsers = futils.renderCheckButton( self.virtualBox, 'Allow virtual users', True, generalFunctions )
        
        # Separator
        futils.renderSeparator( self.anonymousBox )


        self.fill( )

        self.mouseHints = { # Local Users
             self.chAllowLocalUsers  : 'Allow local system users to access the server',
             self.chUsersChmodEnable : 'When enabled, the FTP command SITE CHMOD is allowed for local users. This command allows the users to change the permissions on files.',
             self.chUsersChangeRoot  : 'When enabled, local  users are change-rooted to their home directories after logging in.',
             self.chRootListEnable   : 'When enabled, the local users listed in the file specified below are placed in a chroot jail upon log in.',
             self.eRootListEnable    : 'Specifies the file containing a list of local users which are placed in a chroot jail upon log in.',
             self.chGuestEnable      : 'When enabled, all non-anonymous users are logged in as the user guest, which is the local user specified in the entry below.',
             self.eGuestEnable       : 'Specifies the username the guest user is mapped to.',
             self.eLocalRoot         : 'Specifies the directory vsftpd changes to after a local user logs in.',
             self.eUserConfigDir     : 'Specifies the path to a directory containing configuration files bearing the name of local system users that contain specific settings for that user.',
             self.sLocalMaxRate      : 'Specifies the maximum rate data is transfered for local users logged into the server in bytes per second.',
             self.eLocalUmask        : 'Specifies the umask of created files by local users.',
                            # Anonymous User
             self.chAllowAnonymousUser           : 'Allow anonymous users to access the server.',
             self.chAllowCreatingNewDirectories  : 'When enabled, anonymous users are allowed to create nre directories within a parent directory which has write permissions.',
             self.chAllowUploadFiles             : 'Allow anonymous users to upload files to the server.',
             self.chAskForPassword               : 'When enabled, the anonymous user is not asked for a password.',
             self.chAnonymousWorldOnlyEnable     : 'When enabled, anonymous users are only allowed to download world-readable files.',
             self.chAnonymousUseSSL              : 'Allow anonymous to use SSL',
             self.chAnonymousChOwner             : 'When eabled, all files uploaded by anonymous users are owned by the user specified below.',
             self.eAnonymousChOwner              : 'Specifies the ownership of anonymously uploaded files.',
             self.eAnonymousUserName             : 'Specifies the local user account (listed in /etc/passwd) used for the anonymous FTP user.',
             self.eAnonymousRootDir              : 'Specifies the directory vsftpd changes to after an anonymous user logs in.',
             self.sAnonymousMaxRate              : 'Specifies the maximum data transfer rate for anonymous users in bytes per second.',
             self.eAnonUmask                     : 'Specifies the umas of created files by anonymous user.',
                            # Virtual Users
             self.chAllowVirtualUsers           : 'Allow Virtual users'
        }

        self.enableRootList_toggled( None )
        self.guestEnable_toggled( None )
        self.allowLocalUsers( None )
        self.allowAnonymousUser( None )
        self.chownUploads_toggled( None )

        self.show( mainBox )

    def fill( self ):

        """ Function to fill widgets with values from configuration  """
        # Local Users
        self.chAllowLocalUsers.set_active ( self.config["local_enable"] == "YES" )
        self.chUsersChmodEnable.set_active( self.config["chmod_enable"] == "YES" )
        self.chUsersChangeRoot.set_active ( self.config["chroot_local_user"] == "YES" )
        self.chPasswdChRootEnable.set_active( self.config["passwd_chroot_enable"] == "YES" and self.config["chroot_local_user"] == "YES" )
        self.chRootListEnable.set_active( self.config["chroot_list_enable"] == "YES" )
        self.eRootListEnable.set_text( self.config["chroot_list_file"] )
        self.chGuestEnable.set_active( self.config["guest_enable"] == "YES" )
        self.eGuestEnable.set_text( self.config["guest_username"] )
        if len( self.config["local_root"] ) > 0: self.eLocalRoot.set_text( self.config["local_root"] )
        if len( self.config["user_config_dir"] ) > 0: self.eUserConfigDir.set_text( self.config["user_config_dir"] )
        self.sLocalMaxRate.set_value( float( self.config["local_max_rate"] ) )
        self.eLocalUmask.set_text( self.config["local_umask"] )

        # Anonymous Users
        self.chAllowAnonymousUser.set_active( self.config["anonymous_enable"] == "YES" )
        self.chAllowCreatingNewDirectories.set_active( self.config["anon_mkdir_write_enable"] == "YES" )
        self.chAllowUploadFiles.set_active( self.config["anon_upload_enable"] == "YES" and self.config["write_enable"] == "YES" )
        self.chAskForPassword.set_active( self.config["no_anon_password"] == "YES" )
        self.chAnonymousWorldOnlyEnable.set_active( self.config["anon_world_readable_only"] == "YES" )
        self.chAnonymousUseSSL.set_active( self.config["allow_anon_ssl"] == "YES" )
        self.chAnonymousChOwner.set_active( self.config["chown_uploads"] == "YES" )
        self.eAnonymousChOwner.set_text( self.config["chown_username"]  )
        self.eAnonymousUserName.set_text( self.config["ftp_username"] )
        if len( self.config["anon_root"] ) > 0: self.eAnonymousRootDir.set_text( self.config["anon_root"] )

        self.sAnonymousMaxRate.set_value( float( self.config["anon_max_rate"] ) )
        self.eAnonUmask.set_text( self.config["anon_umask"] )

    def change( self, widget ):
        
        """ Function called when user change some value on the form """
        # Local users
        if widget == self.chUsersChmodEnable: 
            if widget.get_active( ) == True: self.config["chmod_enable"] = "YES"
            else: self.config["chmod_enable"] = "NO"
        if widget == self.chUsersChangeRoot: 
            if widget.get_active( ) == True: self.config["chroot_local_user"] = "YES"
            else: self.config["chroot_local_user"] = "NO"
        if widget == self.chPasswdChRootEnable: 
            if widget.get_active( ) == True: 
                self.config["passwd_chroot_enable"] = "YES"
                self.config["chroot_local_user"] = "YES"
            else: self.config["passwd_chroot_enable"] = "NO"
        if widget == self.eRootListEnable: 
            self.config["chroot_list_file"] = widget.get_text( )
        if widget == self.eGuestEnable: 
            self.config["guest_username"] = widget.get_text( )
        if widget == self.eLocalRoot: 
            self.config["local_root"] = widget.get_text( )
        if widget == self.eUserConfigDir: 
            self.config["user_config_dir"] = widget.get_text( )
        if widget == self.sLocalMaxRate: 
            self.config["local_max_rate"] = `int( widget.get_value( ) )`
        if widget == self.eLocalUmask: # TODO chekck correctness of octal or decimal string
            self.config["local_umask"] = widget.get_text( )

        # Anonymous User
        if widget == self.chAllowCreatingNewDirectories: 
            if widget.get_active( ) == True: self.config["anon_mkdir_write_enable"] = "YES"
            else: self.config["anon_mkdir_write_enable"] = "NO"
        if widget == self.chAllowUploadFiles: 
            if widget.get_active( ) == True: 
		self.config["anon_upload_enable"] = "YES"
		self.config["write_enable"] = "YES"
            else: self.config["anon_upload_enable"] = "NO"
        if widget == self.chAskForPassword: 
            if widget.get_active( ) == True: self.config["no_anon_password"] = "YES"
            else: self.config["no_anon_password"] = "NO"
        if widget == self.chAnonymousWorldOnlyEnable: 
            if widget.get_active( ) == True: self.config["anon_world_readable_only"] = "YES"
            else: self.config["anon_world_readable_only"] = "NO"
        if widget == self.chAnonymousUseSSL: 
            if widget.get_active( ) == True: self.config["allow_anon_ssl"] = "YES"
            else: self.config["allow_anon_ssl"] = "NO"
        if widget == self.eAnonymousChOwner: 
            self.config["chown_username"] = widget.get_text( )
        if widget == self.eAnonymousUserName: 
            self.config["ftp_username"] = widget.get_text( )
        if widget == self.eAnonymousRootDir: 
            self.config["anon_root"] = widget.get_text( )
        if widget == self.sAnonymousMaxRate: 
            self.config["anon_max_rate"] = `int( widget.get_value( ) )`
        if widget == self.eAnonUmask:           # TODO chekck correctness of octal or decimal string
            self.config["anon_umask"] = widget.get_text( )

        # Change
        if widget == self.bRootListEnable:  
            choice = futils.openFile( )
            if choice: self.eRootListEnable.set_text( choice )
        elif widget == self.bLocalRoot or widget == self.bUserConfigDir or widget == self.bAnonymousRootDir: 
            choice = futils.openDir( )
        if   widget == self.bLocalRoot and choice:        self.eLocalRoot.set_text( choice )
        elif widget == self.bUserConfigDir and choice:    self.eUserConfigDir.set_text( choice )
        elif widget == self.bAnonymousRootDir and choice: self.eAnonymousRootDir.set_text( choice )

    def chownUploads_toggled( self, widget ):
        """Called when checkbox on Change owner get toggled"""
        self.eAnonymousChOwner.set_sensitive( self.chAnonymousChOwner.get_active() )

        # Set Config
        if widget == self.chAnonymousChOwner: 
            if widget.get_active( ) == True: self.config["chown_uploads"] = "YES"
            else: self.config["chown_uploads"] = "NO"


    def enableRootList_toggled( self, widget ):
        """Called when checkbox on enabling root list get toggled"""
        self.eRootListEnable.set_sensitive( self.chRootListEnable.get_active( ) )
        self.bRootListEnable.set_sensitive( self.chRootListEnable.get_active( ) )

        # Set Config
        if widget == self.chRootListEnable: 
            if widget.get_active( ) == True: self.config["chroot_list_enable"] = "YES"
            else: self.config["chroot_list_enable"] = "NO"

    def guestEnable_toggled( self, widget ):
        """Called when checkbox on enabling guest get toggled"""
        self.eGuestEnable.set_sensitive( self.chGuestEnable.get_active() )

        # Set Config
        if widget == self.chGuestEnable: 
            if widget.get_active( ) == True: self.config["guest_enable"] = "YES"
            else: self.config["guest_enable"] = "NO"

    def allowLocalUsers( self, widget ):
        """Called when checkbox on allowing local users get toggled"""
        self.systemUsersBox.set_sensitive( self.chAllowLocalUsers.get_active() )

        # Set config
        if widget == self.chAllowLocalUsers: 
            if widget.get_active( ) == True: self.config["local_enable"] = "YES"
            else: self.config["local_enable"] = "NO"

    def allowAnonymousUser( self, widget ):
        """Called when checkbox on allowing anon users get toggled"""
        self.anonymousUserBox.set_sensitive( self.chAllowAnonymousUser.get_active() )

        # Set config
        if widget == self.chAllowAnonymousUser: 
            if widget.get_active( ) == True: self.config["anonymous_enable"] = "YES"
            else: self.config["anonymous_enable"] = "NO"

class DirectoryOptions( renderMainWindow ):

    """
        Class that describes functions for render "Directory Options" window
    """

    def __init__( self, mainBox, config, appbar ): 

        """ Constructor that render all widgets  """
        self.config = config
        self.appbar = appbar

        #path="ico/"
        path="/usr/share/pixmaps/system-config-vsftpd/ico/"

        # render main window from click on the left side
        futils.renderMainLabel( mainBox, path+"directoryH.png", 'Directory Options', 'The following is a list of directives which affect directories' )

        generalFunctions = [ self.change, self.mouseOver, self.mouseEnd ]

        # ------------------------------------------------------------------------------------- #
        # frame with label - Using TCP wrappers

        self.directoryBox = futils.renderVBox( mainBox, 10, 0, 0, 0 )

        # [dirlist_enable]
        self.chAllowViewDirectories = futils.renderCheckButton( None, 'Allow users to view directory lists', False, generalFunctions )

        # [use_localtime]
        alignment1 = gtk.Alignment( 0, 0, 1, 0  )
        alignment1.set_padding( 0, 0, 0, 0 )
        self.chLocalTime = futils.renderCheckButton( None, 'Use localtime', False, generalFunctions )
        alignment1.add( self.chLocalTime )
        self.lLocalTime = futils.renderLabel( None, 'When enabled, directory listings reveal the local time for the\n'
                                                    'computer instead of GTM.', False, 10, 0 ) 
        alignment1.show( )

        # [force_fot_files]
        alignment2 = gtk.Alignment( 0, 0, 1, 0  )
        alignment2.set_padding( 0, 0, 0, 0 )
        self.chForceDotFiles = futils.renderCheckButton( None, 'List files beginning with a dot', False, generalFunctions ) 
        alignment2.add( self.chForceDotFiles )
        self.lForceDotFiles = futils.renderLabel( None, 'When enabled, files beginning with a dot (.) are \n'
                                                        'listed in directory listings, with exception of the . and .. files.', False, 10, 0 )
        alignment2.show( )

        # [hide_ids]
        self.chHideIDS = futils.renderCheckButton( None, "Show 'ftp' as the user and group of each file", False, generalFunctions ) 

        # [text_userdb_names]
        alignment3 = gtk.Alignment( 0, 0, 1, 0  )
        alignment3.set_padding( 0, 0, 0, 0 )
        self.chTextUserDBNames = futils.renderCheckButton( None, 'Enable using text names for UID and GID', False, generalFunctions )
        alignment3.add( self.chTextUserDBNames )
        self.lTextUserDBNames = futils.renderLabel( None, 'By default, numeric IDs are shown in the user and group fields\n'
                                                          'of directory listings. You can get textual names by enabling \n'
                                                          'this parameter. It is off by default for  performance reasons.', False, 10, 0 ) 
        alignment3.show( )

        # [dirmessage_enable]
        alignment4 = gtk.Alignment( 0, 0, 1, 0  )
        alignment4.set_padding( 0, 0, 0, 0 )
        self.chMessagedisplay = futils.renderCheckButton( None, 'Enable display messages', False, generalFunctions )
        alignment4.add( self.chMessagedisplay )
        self.lMessagedisplay = futils.renderLabel( None, 'Enable a message to be displayed whenever a user enters \n'
                                                         'a directory with a message file. This message is \n'
                                                         'found whithin the directory being entered. The name of this\n'
                                                         'file is specified below. ', False, 10, 0 )
        alignment4.show( )

        # Entry with box to define anon root directory
        futils.renderHBox( self.directoryBox, 5, 5, 0, 0 )
        box = futils.renderHBox( self.directoryBox, 0, 0, 0, 0 )

        # [message_file]
        label1 = futils.renderLabel( None, 'Message files:   ', False, 0, 0 ) 
        label1.set_alignment( 0, 0.5 )
        self.eMessageFile = futils.renderEntry( None, 30, generalFunctions )
        self.lMessageFile = futils.renderLabel( None, '\nSpecifies the name of the message\n'
                                                      'files when using the <b>dirmesage_enable</b>\n'
                                                      'directive.', True, 10, 0 ) # TODO - make sensitive false, if option above is disabled

        # [deny_file]
        label2 = futils.renderLabel( None, 'Deny file:   ', False, 0, 0 ) 
        label2.set_alignment( 0, 0.5 )
        self.eDenyFile = futils.renderEntry( None, 30, generalFunctions )
        self.lDenyFile = futils.renderLabel( None, 'Set a pattern for filenames (and directory\n'
                                                   'names etc.) which should not be accessible\n'
                                                   'in any way.', True, 10, 0 )

        # [hide_file]
        label3 = futils.renderLabel( None, 'Hide file:   ', False, 0, 0 ) 
        label3.set_alignment( 0, 0.5 )
        self.eHideFile = futils.renderEntry( None, 30, generalFunctions )
        self.lHideFile = futils.renderLabel( None, 'Set a pattern for filenames (and directory\n'
                                                   'names etc.) which should be hidden from directory\n'
                                                   'listings, but fully accessible to who know what\n'
                                                   'names to actually use', True, 10, 0 )

        
        tableModel = [  [ self.chAllowViewDirectories ],
                        [ futils.renderSeparator() ],
                        [ alignment1, self.lLocalTime ],
                        [ futils.renderSeparator() ],
                        [ alignment2, self.lForceDotFiles ],
                        [ futils.renderSeparator() ],
                        [ self.chHideIDS ],
                        [ futils.renderSeparator() ],
                        [ alignment3, self.lTextUserDBNames ],
                        [ futils.renderSeparator() ],
                        [ alignment4, self.lMessagedisplay ],
                        [ futils.renderSeparator() ],
                        [ label1, self.eMessageFile, gtk.Alignment(0,0.5,0,0), self.lMessageFile],
                        [ futils.renderSeparator() ],
                        [ label2, self.eDenyFile, gtk.Alignment(0,0.5,0,0), self.lDenyFile ],
                        [ futils.renderSeparator() ],
                        [ label3, self.eHideFile, gtk.Alignment(0,0.5,0,0), self.lHideFile ] ]

        futils.renderTable( self.directoryBox, tableModel )
        
        self.mouseHints = {
             self.chAllowViewDirectories    : 'When enabled, users are allowed to view directory lists.',
             self.chLocalTime               : 'When enabled, directory listings reveal the local time for the computer instead of GMT',
             self.chForceDotFiles           : 'When enabled, files beginning with a dot (.) are listed in directory listings, with the exception of the . and .. files.',
             self.chHideIDS                 : 'When enabled, all directory listings show ftp as the user and group for each file.',
             self.chTextUserDBNames         : 'When enabled, test usernames and group names are used in place of UID and GID entries. Enabling this option may slow performance of the server.',
             self.chMessagedisplay          : 'When enabled, a message is displayed whenever a user enters a directory with a message file specified below as "Message files:"',
             self.eMessageFile              : 'Specifies the name of the message file:',
             self.eDenyFile                 : 'This option can be used to set a pattern for filenames (and directory names etc.) which should not be accessible in any way.',
             self.eHideFile                 : 'This option can be used to set a pattern for filenames (and directory names etc.) which should be hidden from directory listings.'
        }

        self.fill( )

        self.show( mainBox )

    def fill( self ):

        """ Function to fill widgets with values from configuration  """
        self.chAllowViewDirectories.set_active( self.config["dirlist_enable"] == "YES" )
        self.chLocalTime.set_active( self.config["use_localtime"] == "YES" )
        self.chForceDotFiles.set_active( self.config["force_dot_files"] == "YES" )
        self.chHideIDS.set_active( self.config["hide_ids"] == "YES" )
        self.chTextUserDBNames.set_active( self.config["text_userdb_names"] == "YES" )
        self.chMessagedisplay.set_active( self.config["dirmessage_enable"] == "YES" )
        self.eMessageFile.set_text( self.config["message_file"] )
        self.eDenyFile.set_text( self.config["deny_file"][ 1:-1] )
        self.eHideFile.set_text( self.config["hide_file"][ 1:-1] )

    def change( self, widget ):

        """ Function called when user change some value on the form """
        if   widget == self.chAllowViewDirectories: 
            if widget.get_active( ) == True: self.config["dirlist_enable"] = "YES"
            else: self.config["dirlist_enable"] = "NO"
        elif widget == self.chLocalTime: 
            if widget.get_active( ) == True: self.config["use_localtime"] = "YES"
            else: self.config["use_localtime"] = "NO"
        elif widget == self.chForceDotFiles: 
            if widget.get_active( ) == True: self.config["force_dot_files"] = "YES"
            else: self.config["force_dot_files"] = "NO"
        elif widget == self.chHideIDS: 
            if widget.get_active( ) == True: self.config["hide_ids"] = "YES"
            else: self.config["hide_ids"] = "NO"
        elif widget == self.chTextUserDBNames: 
            if widget.get_active( ) == True: self.config["text_userdb_names"] = "YES"
            else: self.config["text_userdb_names"] = "NO"
        elif widget == self.chMessagedisplay: 
            if widget.get_active( ) == True: self.config["dirmessage_enable"] = "YES"
            else: self.config["dirmessage_enable"] = "NO"
        elif widget == self.eMessageFile: 
            self.config["message_file"] = widget.get_text( )
        elif widget == self.eDenyFile: 
            self.config["deny_file"] = '{' + widget.get_text( ) + '}'
        elif widget == self.eHideFile: 
            self.config["hide_file"] = '{' + widget.get_text( ) + '}'
        


class Logging( renderMainWindow ):

    """
        Class that describes functions for render "Logging" window
    """

    def __init__( self, mainBox, config, appbar): 

        """ Constructor that render all widgets  """
        self.config = config
        self.appbar = appbar

        #path="ico/"
        path="/usr/share/pixmaps/system-config-vsftpd/ico/"

        # render main window from click on the left side
        futils.renderMainLabel( mainBox, path+"loggingH.png", 'Logging', 'The following is a list of directives which affect vsftpd\'s logging behavior.' )

        # ------------------------------------------------------------------------------------- #
        # frame with label - Using TCP wrappers

        # Box for rendering system box
        box = gtk.VBox( False, 0 )
        mainBox.pack_start( box, False, False, 0 )
        box.show( )
        self.logBox = futils.renderVBox( box, 10, 10, 10, 10 )

        generalFunctions = [ self.change, self.mouseOver, self.mouseEnd ]
        
        # [dual_log_enable]
        self.chAllowDualLog = futils.renderCheckButton( self.logBox, 'Enable dual looging', True, generalFunctions )

        # [log_ftp_protocol]
        self.chLogFTPProtocol = futils.renderCheckButton( self.logBox, 'Log all FTP commands and respones (useful for debugging)', False, generalFunctions )

        # [syslog_enable]
        self.chSystemLog = futils.renderCheckButton( self.logBox, 'Sent all standard log messages to system logger', False, generalFunctions )

        # [xferlog_enable]
        self.chXFerlogEnable = futils.renderCheckButton( self.logBox, 'Log connections and file transfer information', True, generalFunctions )

        # [xferlog_std_format]
        self.chXFerlogStandardFormat = futils.renderCheckButton( self.logBox, 'Use standard format for wu-compatible log file', True, generalFunctions )

        # Separator
        futils.renderHBox( self.logBox, 5, 5, 0, 0 )
        futils.renderSeparator( self.logBox )
               
        # [vsftpd_log_file]
        self.lStandardLogFile = futils.renderLabel( self.logBox, 'Specify the standard log file:', True, 0, 0 )
        # Entry with box with path to file
        box1 = futils.renderHBox( self.logBox, 0, 10, 10, 10 )
        self.eStandardLogFile = futils.renderEntry( box1, 0, generalFunctions )
        self.bStandardLogFile = futils.renderButton( box1, 'Change...', None, False, self.change )

        # [xferlog_file]
        self.lStandardXFerLogFile = futils.renderLabel( self.logBox, 'Specify the wu-ftpd-compatible file:', True, 0, 0 )
        # Entry with box with path to file
        box1 = futils.renderHBox( self.logBox, 0, 10, 10, 10 )
        self.eStandardXFerLogFile = futils.renderEntry( box1, 0, generalFunctions )
        self.bStandardXFerLogFile = futils.renderButton( box1, 'Change...', None, False, self.change )

        self.logBox.show( )

        self.mouseHints = {
             self.chAllowDualLog            : 'When enabled, vsftpd writes two files simultaneously: wu-ftpd compatible log and standard file (both specified below).',
             self.chLogFTPProtocol          : 'When enabled (without wu-compatible log file), all FTP commands and responses are logged. (Useful for debugging).',
             self.chSystemLog               : 'When enabled, all logging normally written to the standard vsftpd log.',
             self.chXFerlogEnable           : 'When enabled, vsftpd logs connections and file transfer information are logged to the standard log and connections to wu-ftpd compatible log.',
             self.chXFerlogStandardFormat   : 'When enabled, only a wu-ftpd-compatible file transfer log is written to the log. This file only logs file transfers.',
             self.eStandardLogFile          : 'Specifies the vsftpd log file. To use this file, beware that system log is disabled...',
             self.eStandardXFerLogFile      : 'Specifies the wu-ftpd-compatible log file.'
        }

        self.fill( )

        self.show( mainBox )

    def fill( self ):

        """ Function to fill widgets with values from configuration  """
        self.chAllowDualLog.set_active( self.config["dual_log_enable"] == "YES" and self.config["xferlog_enable"] == "YES" )
        self.chLogFTPProtocol.set_active( self.config["log_ftp_protocol"] == "YES" and self.config["xferlog_enable"] == "YES" and self.config["xferlog_std_format"] == "NO" )
        self.chSystemLog.set_active( self.config["syslog_enable"] == "YES" and self.config["xferlog_enable"] == "YES" )
        self.chXFerlogEnable.set_active( self.config["xferlog_enable"] == "YES" )
        self.chXFerlogStandardFormat.set_active( self.config["xferlog_std_format"] == "YES" and self.config["xferlog_enable"] == "YES" )
        self.eStandardLogFile.set_text( self.config["vsftpd_log_file"] )
        self.eStandardXFerLogFile.set_text( self.config["xferlog_file"] )

    def change( self, widget ):

        """ Function called when user change some value on the form """
        if widget == self.chAllowDualLog: 
            if widget.get_active( ) == True: 
                self.config["dual_log_enable"] = "YES"
                self.config["xferlog_enable"] = "YES"
            else: self.config["dual_log_enable"] = "NO"
        if widget == self.chLogFTPProtocol: 
            if widget.get_active( ) == True: 
                self.config["log_ftp_protocol"] = "YES"
                self.config["xferlog_enable"] = "YES"
                self.config["xferlog_std_format"] = "NO"
            else: self.config["log_ftp_protocol"] = "NO"
        if widget == self.chSystemLog: 
            if widget.get_active( ) == True: 
                self.config["syslog_enable"] = "YES"
                self.config["xferlog_enable"] = "YES"
            else: self.config["syslog_enable"] = "NO"
        if widget == self.chXFerlogEnable: 
            if widget.get_active( ) == True: self.config["xferlog_enable"] = "YES"
            else: self.config["xferlog_enable"] = "NO"
        if widget == self.chXFerlogStandardFormat: 
            if widget.get_active( ) == True: 
                self.config["xferlog_std_format"] = "YES"
                self.config["xferlog_enable"] = "YES"
            else: self.config["xferlog_std_format"] = "NO"
        if widget == self.eStandardLogFile: 
            self.config["vsftpd_log_file"] = widget.get_text( )
        if widget == self.eStandardXFerLogFile: 
            self.config["xferlog_file"] = widget.get_text( )

        if widget in [ self.bStandardLogFile, self.bStandardXFerLogFile ]:
            choice = futils.openFile( )
        if   widget == self.bStandardLogFile and choice:        self.eStandardLogFile.set_text( choice )
        elif widget == self.bStandardXFerLogFile and choice:    self.eStandardXFerLogFile.set_text( choice )

        self.fill( )

class NetworkOptions( renderMainWindow ):

    """
        Class that describes functions for render "Network Options" window
    """

    def __init__( self, mainBox, config, appbar ): 

        """ Constructor that render all widgets  """
        self.config = config
        self.appbar = appbar

        #path="ico/"
        path="/usr/share/pixmaps/system-config-vsftpd/ico/"

        # render main window from click on the left side
        futils.renderMainLabel( mainBox, path+"networkH.png", 'Network Options', 'The following is a list of directives which affect how vsftpd interacts whith the network' )

        # ---------------------------------------------------------------------------------------- #

        generalFunctions = [ self.change, self.mouseOver, self.mouseEnd ]

        self.networkNotebook = gtk.Notebook( )
        self.networkNotebook.set_tab_pos( gtk.POS_TOP )
        self.networkNotebook.show( )

        ( alignment, self.networkBox) = futils.renderVBox( None, 10, 10, 10, 10 )
        mainBox.pack_start( alignment, True, True, 0 )

        # Labels for notebook
        alabel = gtk.Label( 'Active connection' )
        alabel.show( )
        plabel = gtk.Label( 'Passive connection' )
        plabel.show( )

        # ------------------------------------------------------------------------------------- #
        # General

        # [connect_from_port_20]
        self.chConnectFromPort20 = futils.renderCheckButton( self.networkBox, 'Run vsftpd with enough privileges to open port 20 on the server '
                                                                              'during active mode data transfers', True, generalFunctions )

        label = futils.renderLabel( self.networkBox, '<i>Disabling this option allows vsftpd to run with less privileges, but may be incompatible '
                                                     'with some FTP clients</i>', True, 0, 0 )


        # horizontal separator
        futils.renderHBox( self.networkBox, 5, 5, 0, 0 )

        box = futils.renderHBox( self.networkBox, 0, 0, 0, 0 )

        # [data_connection_timeout]
        self.lDataConTimeOut = futils.renderLabel( None, 'Maximum amount of time to stall the data transfer:   ', False, 0, 0 ) 
        self.lDataConTimeOut.set_alignment( 0, 0.5 )
        self.sDataConTimeOut = futils.renderSpin( None, 300.0, 0.0, 3600.0, 1.0, 100.0, 0.0, 0, 0, generalFunctions )
        self.lDataConTimeOut2 = futils.renderLabel( None, '   seconds', False, 0, 0 ) 
        self.lDataConTimeOut2.set_alignment( 0, 0.5 )

        # [idle_session_timeout]
        self.lIdleSession = futils.renderLabel( None, 'Maximum amount of time between commands from client:   ', False, 0, 0 ) 
        self.lIdleSession.set_alignment( 0, 0.5 )
        self.sIdleSession = futils.renderSpin( None, 300.0, 0.0, 3600.0, 1.0, 100.0, 0.0, 0, 0, generalFunctions )
        self.lIdleSession2 = futils.renderLabel( None, '   seconds', False, 0, 0 ) 
        self.lIdleSession2.set_alignment( 0, 0.5 )

        # [liten_port]
        self.lListenPort = futils.renderLabel( None, 'Port on wich vsftpd listens for network connection:   ', False, 0, 0 ) 
        self.lListenPort.set_alignment( 0, 0.5 )
        self.sListenPort = futils.renderSpin( None, 21.0, 0.0, 65535.0, 1.0, 100.0, 0.0, 0, 0, generalFunctions  )

        # [max_clients]
        self.lMaxClients = futils.renderLabel( None, 'Maximum number of simultaneous clients allowed to connect to the server:   ', False, 0, 0 ) 
        self.lMaxClients.set_alignment( 0, 0.5 )
        self.sMaxClients = futils.renderSpin( None, 21.0, 0.0, 9999.0, 1.0, 100.0, 0.0, 0, 0, generalFunctions )

        # [max_per_ip]
        self.lMaxPerIP = futils.renderLabel( None, 'Maximum of clients allowed to connect from the same source IP address:   ', False, 0, 0 ) 
        self.lMaxPerIP.set_alignment( 0, 0.5 )
        self.sMaxPerIP = futils.renderSpin( None, 0.0, 0.0, 999.0, 1.0, 100.0, 0.0, 0, 0, generalFunctions )


        tableModel = [ [ self.lDataConTimeOut, self.sDataConTimeOut, self.lDataConTimeOut2 ],
                       [ self.lIdleSession, self.sIdleSession, self.lIdleSession2 ],
                       [ self.lListenPort, self.sListenPort, None ],
                       [ self.lMaxClients, self.sMaxClients, None ],
                       [ self.lMaxPerIP, self.sMaxPerIP, None ] ]

        futils.renderTable( box, tableModel )

        # horizontal separator
        futils.renderHBox( self.networkBox, 5, 5, 0, 0 )
        # Separator
        futils.renderSeparator( self.networkBox )
        # horizontal separator
        futils.renderHBox( self.networkBox, 5, 5, 0, 0 )
               
        self.networkBox.pack_start( self.networkNotebook, True, True, 0 )


        # ------------------------------------------------------------------------------------- #
        # Active connection

        """ 
        I did to boxes, first anonymousBox, where is checkbox to allow other options
        and second, where I take other options. It is simple to set sensitivity to al options
        in one command 
        """
        # Box for rendering system box
        box = gtk.VBox( False, 0 ) 
        box.show( )
        self.activeBox = futils.renderVBox( box, 10, 10, 10, 10 )
        self.networkNotebook.append_page( box, alabel )

        futils.renderLabel( self.activeBox, "<i>Active connection: Data are sending throw same port as commands</i>", True, 0, 0 )
        
        # horizontal separator
        futils.renderHBox( self.activeBox, 5, 5, 0, 0 )

        # [port_enable]
        self.chAllowActive = futils.renderCheckButton( self.activeBox, 'Enable active connections', True, [self.allowActiveConnection_toggled] + generalFunctions[1:] )
        
        # horizontal separator
        futils.renderHBox( self.activeBox, 5, 5, 0, 0 )

        box = futils.renderHBox( self.activeBox, 0, 0, 0, 0 )

        # [connect_timeout]
        self.lAConnectionTimeOut = futils.renderLabel( None, 'Maximum amount of time a client has to respond to a data connection:   ', False, 0, 0 ) 
        self.lAConnectionTimeOut.set_alignment( 0, 0.5 )
        self.sAConnectionTimeOut = futils.renderSpin( None, 300.0, 0.0, 3600.0, 1.0, 100.0, 0.0, 0, 0, generalFunctions )
        self.lAConnectionTimeOut2 = futils.renderLabel( None, '   seconds', False, 0, 0 ) 
        label.set_alignment( 0, 0.5 )

        # [ftp_data_port]
        self.lADataPort = futils.renderLabel( None, 'Port used for active data connections:   ', False, 0, 0 ) 
        self.lADataPort.set_alignment( 0, 0.5 )
        self.sADataPort = futils.renderSpin( None, 20.0, 0.0, 65535.0, 1.0, 100.0, 0.0, 0, 0, generalFunctions )


        tableModel = [ [ self.lAConnectionTimeOut, self.sAConnectionTimeOut, self.lAConnectionTimeOut2 ],
                       [ self.lADataPort, self.sADataPort, None ] ]

        futils.renderTable( box, tableModel )

        # ------------------------------------------------------------------------------------- #
        # Passive connection

        """ 
        I did to boxes, first anonymousBox, where is checkbox to allow other options
        and second, where I take other options. It is simple to set sensitivity to al options
        in one command 
        """
        # Box for rendering system box
        box = gtk.VBox( False, 0 ) 
        box.show( )
        self.passiveBox = futils.renderVBox( box, 10, 10, 10, 10 )
        self.networkNotebook.append_page( box, plabel )

        futils.renderLabel( self.passiveBox, "<i>Passive connection: Data are sending throw different ports as commands</i>", True, 0, 0 )
        
        # horizontal separator
        futils.renderHBox( self.passiveBox, 5, 5, 0, 0 )

        # [pasv_enable]
        self.chAllowPassive = futils.renderCheckButton( self.passiveBox, 'Enable passive connections', True, [self.allowPassiveConnection_toggled] + generalFunctions[1:] )
        
        # horizontal separator
        futils.renderHBox( self.passiveBox, 5, 5, 0, 0 )

        box = futils.renderHBox( self.passiveBox, 0, 0, 0, 0 )

        # [accept_timeout]
        self.lPAcceptTimeOut = futils.renderLabel( None, 'Maximum amount of time for a client to establish a connection:   ', False, 0, 0 ) 
        self.lPAcceptTimeOut.set_alignment( 0, 0.5 )
        self.sPAcceptTimeOut = futils.renderSpin( None, 300.0, 0.0, 3600.0, 1.0, 100.0, 0.0, 0, 0, generalFunctions )
        self.lPAcceptTimeOut2 = futils.renderLabel( None, '   seconds', False, 0, 0 ) 
        self.lPAcceptTimeOut2.set_alignment( 0, 0.5 )

        # [pasv_max_port] [pasv_min_port]
        self.lPPorts = futils.renderLabel( None, 'Range of ports to bind by passive data transfer:   ', False, 0, 0 ) 
        self.lPPorts.set_alignment( 0, 0.5 )
        self.sPMinPort = futils.renderSpin( None, 1024.0, 1024.0, 65535.0, 1.0, 100.0, 0.0, 0, 0, generalFunctions )
        self.lPPorts2 = futils.renderLabel( None, ' - ', False, 0, 0 ) 
        self.lPPorts2.set_alignment( 0, 0.5 )
        self.sPMaxPort = futils.renderSpin( None, 65535.0, 1024.0, 65535.0, 1.0, 100.0, 0.0, 0, 0, generalFunctions )

        tableModel = [ [ self.lPAcceptTimeOut, self.sPAcceptTimeOut, None, self.lPAcceptTimeOut2 ],
                       [ self.lPPorts, self.sPMinPort, self.lPPorts2, self.sPMaxPort ] ]

        futils.renderTable( box, tableModel )

        # horizontal separator
        futils.renderHBox( self.passiveBox, 5, 5, 0, 0 )

        # CheckButton 
        self.chPPasvPromiscuous = futils.renderCheckButton( self.passiveBox, 'Check to make sure, that data connections are originating from the same IP', True, generalFunctions )

        self.mouseHints = {
             self.chConnectFromPort20   : 'When enabled, vsftpd runs with enough privileges to open port 20 on the server during active mode data transfers.',
             self.sDataConTimeOut       : 'Specifies maximum amount of time data transfers are allowed to stall, in seconds.',
             self.sIdleSession          : 'Specifies the maximum amount of time between commands from a remote client.',
             self.sListenPort           : 'Spcifies the port on which vsftpd listens for network connections.',
             self.sMaxClients           : 'Specifies the maximum number of simultaneous clients allowed to connect to the server when it is running in standalone mode.',
             self.sMaxPerIP             : 'Specifies the maximum of clients allowed to connected from the same source IP address',
             self.chAllowActive         : 'When enabled, active mode connects are allowed.',
             self.sAConnectionTimeOut   : 'Specifies maximum amount of time a client using active mode has to respond to a data connection, in seconds.',
             self.sADataPort            : 'Specifies the port used for active data connections.',
             self.chAllowPassive        : 'When enabled, passive mode connects are allowed.',
             self.sPAcceptTimeOut       : 'Specifies the amount of time for a client using passive mode to establish a connection.',
             self.sPMinPort             : 'Specifies the lowest possible port sent to the FTP clients for passive mode connections.',
             self.sPMaxPort             : 'Specifies the highest possible port sent to the FTP clients for passive mode connections.'
        }


        self.fill( )

        self.show( mainBox )
        
    def fill( self ):

        """ Function to fill widgets with values from configuration  """
        # General
        self.chConnectFromPort20.set_active( self.config["connect_from_port_20"] == "YES" )
        self.sDataConTimeOut.set_value( float( self.config["data_connection_timeout"] ) )
        self.sIdleSession.set_value( float( self.config["idle_session_timeout"] ) )
        self.sListenPort.set_value( float( self.config["listen_port"] ) )
        self.sMaxClients.set_value( float( self.config["max_clients"] ) )
        self.sMaxPerIP.set_value( float( self.config["max_per_ip"] ) )

        # Active
        self.chAllowActive.set_active( self.config["port_enable"] == "YES" )
        self.sAConnectionTimeOut.set_value( float( self.config["connect_timeout"] ) )
        self.sADataPort.set_value( float( self.config["ftp_data_port"] ) )

        # Passive
        self.chAllowPassive.set_active( self.config["pasv_enable"] == "YES" )
        self.sPAcceptTimeOut.set_value( float( self.config["accept_timeout"] ) )
        self.sPMinPort.set_value( float( self.config["pasv_min_port"] ) )
        self.sPMaxPort.set_value( float( self.config["pasv_max_port"] ) )
        self.chPPasvPromiscuous.set_active( self.config["pasv_promiscuous"] == "YES" )

    def change( self, widget ):

        """ Function called when user change some value on the form """
        if widget == self.chConnectFromPort20: 
            if widget.get_active( ) == True: self.config["connect_from_port_20"] = "YES"
            else: self.config["connect_from_port_20"] = "NO"
        elif widget == self.sDataConTimeOut: 
            self.config["data_connection_timeout"] = `int( widget.get_value( ) )`
        elif widget == self.sIdleSession: 
            self.config["idle_session_timeout"] = `int ( widget.get_value( ) )`
        elif widget == self.sListenPort: 
            self.config["listen_port"] = `int( widget.get_value( ) )`
        elif widget == self.sMaxClients: 
            self.config["max_clients"] = `int( widget.get_value( ) )`
        elif widget == self.sMaxPerIP: 
            self.config["max_per_ip"] = `int( widget.get_value( ) )`

        # Active
        elif widget == self.sAConnectionTimeOut: 
            self.config["connect_timeout"] = `int( widget.get_value( ) )`
        elif widget == self.sADataPort: 
            self.config["ftp_data_port"] = `int( widget.get_value( ) )`

        # Passive
        elif widget == self.sPAcceptTimeOut: 
            self.config["accept_timeout"] = `int( widget.get_value( ) )`
        elif widget == self.sPMinPort: 
            if int( widget.get_value( ) ) == 1024: 
                self.config["pasv_min_port"] = `0`
            else:
                self.config["pasv_min_port"] = `int( widget.get_value( ) )`
        elif widget == self.sPMaxPort: 
            if int( widget.get_value( ) ) == 65535: 
                self.config["pasv_max_port"] = `0`
            else:
                self.config["pasv_max_port"] = `int( widget.get_value( ) )`
        elif widget == self.chPPasvPromiscuous: 
            if widget.get_active( ) == True: self.config["pasv_promiscuous"] = "YES"
            else: self.config["pasv_promiscuous"] = "NO"

    def allowActiveConnection_toggled( self, widget ):
        """Called when checkbox on allowing active connections get toggled"""
        self.lAConnectionTimeOut.set_sensitive ( self.chAllowActive.get_active( ) )
        self.lAConnectionTimeOut2.set_sensitive( self.chAllowActive.get_active( ) )
        self.sAConnectionTimeOut.set_sensitive ( self.chAllowActive.get_active( ) )
        self.lADataPort.set_sensitive( self.chAllowActive.get_active( ) )
        self.sADataPort.set_sensitive ( self.chAllowActive.get_active( ) )

        # Set Config
        if widget.get_active( ) == True: self.config["port_enable"] = "YES"
        else: self.config["port_enable"] = "NO"

    def allowPassiveConnection_toggled( self, widget ):
        """Called when checkbox on allowing passive connections get toggled"""
        self.lPAcceptTimeOut.set_sensitive ( self.chAllowPassive.get_active( ) )
        self.lPAcceptTimeOut2.set_sensitive( self.chAllowPassive.get_active( ) )
        self.sPAcceptTimeOut.set_sensitive( self.chAllowPassive.get_active( ) )
        self.lPPorts.set_sensitive ( self.chAllowPassive.get_active( ) )
        self.lPPorts2.set_sensitive( self.chAllowPassive.get_active( ) )
        self.sPMinPort.set_sensitive( self.chAllowPassive.get_active( ) )
        self.sPMaxPort.set_sensitive( self.chAllowPassive.get_active( ) )
        self.chPPasvPromiscuous.set_sensitive( self.chAllowPassive.get_active( ) )

        # Set Config
        if widget.get_active( ) == True: self.config["pasv_enable"] = "YES"
        else: self.config["pasv_enable"] = "NO"

class ExperimentalLog( renderMainWindow ):

    """
        Class that describes functions for render "Transfer Log" window
    """

    def __init__( self, mainBox, config, appbar ): 

        """ Constructor that render all widgets  """
        self.config = config
        self.appbar = appbar

        #path="ico/"
        path="/usr/share/pixmaps/system-config-vsftpd/ico/"

        # render main window from click on the left side
        futils.renderMainLabel( mainBox, path+"logH.png", 'Transfer log', 'Read only xferlog in graphical format' )

        # ---------------------------------------------------------------------------------------- #

        #self.appbar.set_status( "Please wait, parsing configuration,..." )
        generalFunctions = [ None, self.mouseOver, self.mouseEnd ]

        # generating box
        boxLT = futils.renderHBox( mainBox, 0, 0, 0, 0 )

        # generating calendar
        self.calendar = gtk.Calendar( )
        self.calendar.connect("month_changed", self.calendar_toggled)
        self.calendar.connect("day_selected", self.calendar_toggled)
        self.calendar.connect("day_selected_double_click", self.calendar_toggled)
        self.calendar.connect("prev_month", self.calendar_update)
        self.calendar.connect("next_month", self.calendar_update)
        self.calendar.connect("prev_year", self.calendar_update)
        self.calendar.connect("next_year", self.calendar_update)

        boxLT.pack_start( self.calendar, False, False, 0 )
        self.calendar.show( )

        futils.renderVSeparator( boxLT )

        self.IPscWindow = gtk.ScrolledWindow( )
        self.IPscWindow.set_border_width( 0 )
        self.IPscWindow.set_policy( gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC )
        self.IPscWindow.show( )

        boxLT.pack_start( self.IPscWindow, True, True, 0 )

        futils.renderSeparator( mainBox )

        # generating scrolled window
        self.scWindow = gtk.ScrolledWindow( )
        self.scWindow.set_border_width( 0 )
        self.scWindow.set_policy( gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC )
        self.scWindow.show( )

        mainBox.pack_start( self.scWindow, True, True, 0 )
        gtk.gdk.flush()

        # generating box for adding ll stuff
        ( alignment, self.logBox) = futils.renderVBox( None, 0, 0, 0, 0 )
        self.scWindow.add_with_viewport( alignment )

        # parsing xferlog
        self.log = logs.Log( self.config )
 
        if self.log.dict == None:
		futils.renderLabel( self.logBox, "<b>Log doesn't exists, try to select one in 'Logging' menu</b>", True, 20, 10 )
                self.show( mainBox )
                return

        if len( self.log.dict.keys() ) == 0:
		futils.renderLabel( self.logBox, "<b>Log is empty</b>", True, 20, 10 )

        self.__fileListStore, self.__fileList = futils.create_File_List( self.logBox )
        self.__IPListStore, self.__IPList   = futils.create_IP_List( self.IPscWindow )
        
        if gtk.gtk_version[0] == 2 and gtk.gtk_version[1] > 10: 
            self.__fileList.set_grid_lines( gtk.TREE_VIEW_GRID_LINES_HORIZONTAL )

        # to know about rows selecting
        treeSelection = self.__IPList.get_selection( )
        treeSelection.connect( 'changed', self.iplist_changed )


        self.calendar_update( None )
        self.calendar_toggled( None )

        self.appbar.set_status( "" )
        self.show( mainBox )
        

    def calendar_toggled( self, widget):
        """When changed day in calendar"""
        year, month, day = self.calendar.get_date()
        date = `day`+'.'+self.log.get_number_month( month+1 )+'.'+`year`
        self.__IPListStore.clear( )
        if date in self.log.dict:
            for ip in self.log.dict[ date ]:
                self.__IPListStore.append( [ip] )
        self.__IPList.get_selection( ).select_path( 0 )

    def calendar_update( self, widget):
        """When changed year or month in calendar"""

        self.calendar.clear_marks( )
        yearc, monthc, dayc = self.calendar.get_date()
        for date in self.log.dict:
            day, month, year = self.log.parse_date( date )
            if ( month, year ) == ( monthc+1, yearc ):
                self.calendar.mark_day( day )
        self.calendar_toggled( None )

    def iplist_changed( self, widget ):
        """When changed focused line in ipList """

        #path="ico/"
        path="/usr/share/pixmaps/system-config-vsftpd/ico/"
        __ico_download = gtk.gdk.pixbuf_new_from_file(path+"download.png")
        __ico_upload = gtk.gdk.pixbuf_new_from_file(path+"upload.png")

        year, month, day = self.calendar.get_date()
        date = `day`+'.'+self.log.get_number_month( month+1 )+'.'+`year`
        self.__fileListStore.clear( )
        ( model, iter ) = widget.get_selected( )
        if iter:
            i = -1
            ip = model.get_value( iter, 0 ) 
            __pLines = self.log.parse( date, ip )
            for __line in __pLines:
                i = i + 1
                # if ? on user identification
                if __line[9][0] == "?" or __line[9][0] == ' ': user = "ftp"
                else: user = __line[9]

                if __line[7][0] == "i": 
                    self.__fileListStore.append( [__line[0],__ico_download,user,`__line[4]`] )
                else: self.__fileListStore.append( [__line[0],__ico_upload,user,`__line[4]`] )

